/*
 * The contents of this file are
 * published under the Mozilla Public License 1.1
 */

//@@@@@@@@@@@@@@@@
function EDITOR_Controller		//<c><a>public<d>HTML editor class
(
_editorId		//<p>string<d>Editor element ID
){
//================
this.EDITOR_Controller = function		//<m>void<a>private<d>class constructor
(
editorId		//<p>string<d>Editor element ID
){
this.className	= 'EDITOR_Controller';
this.editorId	= editorId;
this.set_EDITOR_Controller();
}//</m>EDITOR_Controller



//================
this.set_EDITOR_Controller = function		//<m>void<a>private<d>Instance i12n
(){
this.errorNotifier	= new ERROR_Notifier(this.className);
this.dom			= new DOM_Monitor();
this.editorDoc		= 
this.editorBody		= null;
this.setUp();
}//</m>set_EDITOR_Controller



//================
this.setUp = function		//<m>void<a>private<d>Set up document to edit
(){
try{
	this.editorDoc = this.dom.getElement(this.editorId).contentDocument;
	this.editorDoc.designMode = 'on';
	this.editorBody = this.editorDoc.body;
	this.setContent('', XC.K.gui.REPLACE_CONTENT);
	return this.K.ERROR.NONE;
	}
catch(e){
	return this.errorNotifier.setOut(e, this.K.ERROR.DOC_ACCESS);
	}
}//</m>setUp



//================
this.setContent = function		//<m>int<a>public<d>Set up content in html mode
(
content,		//<p>string<d>Content
replacing		//<p>boolean<d>Replacing content flag: replace || append --optional
){
try{
	var
	/*string*/editorHTML;
	if(replacing) editorHTML = this.getWithEntities(content);
	else{
		var
		/*Object*/range = this.dom.getElement(this.editorId).contentWindow.getSelection().getRangeAt(0);
		if(!range.collapsed) range.deleteContents();
		var
		/*string[]*/contentSet = content.split(/\r?\n/g);
		for(var i=contentSet.length-1, m=i; i>=0; i--){
			if((m > 1) && contentSet[i]) range.insertNode(document.createElement('br'));
			range.insertNode(document.createTextNode(contentSet[i]));
			}
		editorHTML = this.editorBody.innerHTML;
		}
	this.editorBody.innerHTML = editorHTML;	//redraw editor content
	return this.K.ERROR.NONE;
	}
catch(e){
	return this.errorNotifier.setOut(e, this.K.ERROR.SET_CONTENT);
	}
}//</m>setContent



//================
this.setCommand = function		//<m>int<a>public<d>Execute the specified command
(
command,		//<p>string<d>Command name
arg				//<p>string<d>Command argument --optional
){
try{
	this.editorDoc.execCommand(command, this.K.SHOW_UI, arg);
	return this.K.ERROR.NONE;
	}
catch(e){
	return this.errorNotifier.setOut(e, this.K.ERROR.EXEC_COMMAND);
	}
}//</m>setCommand



//================
this.setStyle = function		//<m>int<a>public<d>Setup the specified attribute on editorBody
(
//args		//<p>sstring[][]<d>Style name/style value pairs
){
try{
	var
	/*string*/style = '';
	for(var i=0,m=arguments.length;i<m;i++) style += arguments[i][0] + ':' + arguments[i][1] + ';';
	this.editorBody.setAttribute('style', style);
	return this.K.ERROR.NONE;
	}
catch(e){
	return this.errorNotifier.setOut(e, this.K.ERROR.EXEC_COMMAND);
	}
}//</m>setStyle



//================
this.getContent = function		//<m>void<a>public<d>Get content
(){
try{
	return this.getWithoutEntities(this.editorBody.innerHTML);
	}
catch(e){
	this.errorNotifier.setOut(e, this.K.ERROR.HTML_CONTENT);
	return '';
	}
}//</m>getContent



//================
this.getWithEntities = function		//<m>string<a>private<d>Convert XML special chars to entities
(
content		//<p>string<d>Content
){
try{
	if(content == '') return '';
	var
	/*string*/encoded = content.replace(/&/g, '&#38;');
	encoded = encoded.replace(/</g, '&#60;');
	encoded = encoded.replace(/>/g, '&#62;');
	encoded = encoded.replace(/\x0a/g, '<br>\u000a');
	encoded = encoded.replace(/\t/g, '&#160;&#160;&#160;&#160;');	//indentation
	return encoded;
	}
catch(e){
	return '';
	}
}//</m>getWithEntities



//================
this.getWithoutEntities = function		//<m>string<a>private<d>Convert entities to XML special chars
(
content		//<p>string<d>Content
){
try{
	if(content == '') return '';
	var
	/*string*/encoded = content.replace(/(<br>)+$/, '');
	encoded = encoded.replace(/(<br>)+[\x0a\x0d]*/g, '\u000a');
	encoded = encoded.replace(/[\x0a\x0d]+(&nbsp;)+/g, '\u000a');	//indentation-1
	encoded = encoded.replace(/    /g, '\u0009');					//indentation-2
	encoded = encoded.replace(/<br>\x09/g, '\u000a\u0009');		//indentation-3
	encoded = encoded.replace(/&lt;/g, '<');
	encoded = encoded.replace(/&gt;/g, '>');
	encoded = encoded.replace(/&amp;/g, '&');
	return encoded;
	}
catch(e){
	return '';
	}
}//</m>getWithoutEntities



//================
this.get_editorBody = function		//<m>Object<a>public<d>Get editor body node
(){
return this.editorBody;
}//</m>get_editorBody



this.K =
	{
	SHOW_UI:					false,

	xul:
		{
		},

	ERROR:
		{
		//NON FATAL
		NONE:					0,
		NO_SUCCESS:				1,

		//FATAL
		DOC_ACCESS:				-1,
		EXEC_COMMAND:			-2,
		SET_CONTENT:			-3,
		HTML_CONTENT:			-4,
		FATAL:					0xFF		//end of error codes
		}
	};



this.EDITOR_Controller(_editorId);
}//</c>EDITOR_Controller



